BlobError

Enum BlobError 

pub enum BlobError {
    InvalidHeader,
    UnallocatedId,
    UncommittedIndex,
    InvalidIndex,
}
Possible error conditions when retrieving a blob by ID, either via iteration or get_blob().

Variants



InvalidHeader

The header was invalid (purportedly either with indices extending past the blob head offset or the blob head offset extending past the length of the buffer). Suggests invalid initialization or corruption.



UnallocatedId

The requested ID has not yet been allocated.



UncommittedIndex

Suggests a lost race in which the requested ID is valid and the header has been updated to reflect that, but the bookkeeping index has not yet been committed. Such a case unfortunately is indistinguishable from the other possibility that the index was already committed but then subsequently corrupted with zeroes. Where there is confidence in the first case, this call should be retried.



InvalidIndex

The corresponding bookkeeping index is invalid, with the blob purportedly not being contained within [blob head, end of region). Suggests corruption.