Header

Struct Header 

#[repr(C)]
pub struct Header { pub next_id: u32, pub blob_head: u32, }
Header for the ID allocator region.

Fields

next_id: u32blob_head: u32

Implementations

Source

impl Header

pub const SIZE: usize

pub fn from_u64(val: u64) -> Self

pub fn to_u64(self) -> u64

pub fn index_end(self) -> Option<u32>

Returns the byte offset just past the last bookkeeping index, or None if the computation would overflow a u32.

pub fn is_valid(self, length: usize) -> bool

Returns true if the header reflects a valid state for a buffer of length bytes.

pub fn remaining_bytes(self, length: usize) -> Option<usize>

Returns the remaining number of available bytes in the region, given the length of the region, or None in the event of an invalid header.