Index

Struct Index 

#[repr(C)]
pub struct Index { pub size: u32, pub offset: u32, }
Represents a blob bookkeeping index slot.

Fields

size: u32offset: u32

Implementations

Source

impl Index

pub const SIZE: usize

pub fn from_u64(val: u64) -> Self

pub fn to_u64(self) -> u64

pub fn is_valid(self, blob_head: u32, length: usize) -> bool

See BlobError::InvalidIndex. The current blob head offset and length of the region must be provided, and are assumed to have already been validated.