pub enum AllocateError {
InvalidHeader,
NonEmptyIndex,
OutOfMemory,
}Expand description
Possible error conditions when attempting to allocate a blob ID.
Variants
InvalidHeader
The header was invalid (purportedly either with indices extending past the blob head offset or the blob head offset extending past the length of the buffer). Suggests invalid initialization or corruption.
NonEmptyIndex
The would-be bookkeeping index slot for the new allocated ID is non-empty (i.e., not in the initialized state). Suggests corruption.
OutOfMemory
Out Of Memory: there is insufficient memory available for the requested allocation.
(What is available can be queried with remaining_bytes()).